local bn_assets = require("BattleNetwork.Assets")

local lance_texture = bn_assets.load_texture("bn4_spell_lance.png")
local lance_anim_path = bn_assets.fetch_animation_path("bn4_spell_lance.animation")
local lance_audio = bn_assets.load_audio("sword.ogg")

local track_health;

function card_mutate(user, card_index)
	if Player.from(user) ~= nil then
		local deck = user:deck_cards()
		local count = #deck
		local removed = 0
		local stop_early = false
		while removed < 5 and not stop_early do
			if removed >= count then stop_early = true end
			if stop_early == true then return end

			local card_to_remove = math.random(1, count)
			user:remove_deck_card(card_to_remove)

			removed = removed + 1
			count = #user:deck_cards()
		end
	end

	if track_health == nil then
		track_health = user:create_component(Lifetime.ActiveBattle)
		track_health._stored_value = 0
		track_health._is_update_value = true
		track_health._field_list = user:field():find_characters(function(ent)
			if Living.from(ent) == nil then return false end
			if ent:current_tile() == nil then return false end
			if user:is_team(ent:team()) then return false end
			return true
		end)

		track_health.on_update_func = function(self)
			local owner = self:owner()
			local card = owner:field_card(1)

			if card == nil then return end

			for _, tag in ipairs(card.tags) do
				if tag == "HALF_FOE_HEALTH_EQUALS_POWER" then
					for i = 1, #self._field_list, 1 do
						local value = math.max(1, math.floor(self._field_list[i]:health() / 2))
						if value > self._stored_value then
							self._stored_value = value
						end
					end

					card.damage = math.min(999, self._stored_value)

					user:set_field_card(1, card)
				end
			end
		end
	end
end

local function spawn_spell(tile, props, user)
	local spell = Spell.new(user:team())
	spell:set_hit_props(
		HitProps.from_card(props),
		user:context(),
		Drag.new(tile:facing(), 1)
	)

	spell:set_tile_highlight(Highlight.Solid)

	spell:sprite():set_texture(lance_texture)
	local spell_anim = spell:animation()
	spell_anim:load(lance_anim_path)
	spell_anim:set_state("DARK")

	spell:set_offset(40, 0)

	spell._is_fade = false
	spell._fade_timer = 0
	spell._sprite_ref = spell:sprite()
	spell._flicker_count = 7
	spell:sprite():set_layer(-2)
	spell.on_update_func = function(self)
		if self._flicker_count == 0 then
			self:erase()
			return
		end

		if self._is_fade == true then
			self._fade_timer = self._fade_timer + 1
			if self._fade_timer % 4 == 0 then
				self._sprite_ref:set_visible(not self._sprite_ref:visible())
				self._flicker_count = self._flicker_count - 1
			end
			return
		end

		local offset = self:offset()
		if offset.x > 0 then
			self:set_offset(offset.x - 8, offset.y)
		else
			self._is_fade = true
		end

		self:attack_tile()
	end

	user:field():spawn(spell, tile)
end

function card_init(actor, props)
	local action = Action.new(actor, "CHARACTER_IDLE")

	action:set_lockout(ActionLockout.new_async(40))

	action.on_execute_func = function(self, user)
		local field = user:field()
		local tiles = {
			field:tile_at(6, 1),
			field:tile_at(6, 2),
			field:tile_at(6, 3)
		}

		for _, tile in ipairs(tiles) do
			spawn_spell(tile, props, user)
		end

		Resources.play_audio(lance_audio)
	end
	return action
end
